@@ -1,5 +1,5 @@
 # These functions are
-# Copyright (C) 1998-2013 T.W. Yee, University of Auckland.
+# Copyright (C) 1998-2015 T.W. Yee, University of Auckland.
 # All rights reserved.
 
 
@@ -15,9 +15,8 @@ vlm <- function(formula,
                 contrasts = NULL, 
                 constraints = NULL,
                 extra = NULL, offset = NULL,  
-                smart = TRUE, ...)
-{
-  dataname <- as.character(substitute(data)) # "list" if no data=
+                smart = TRUE, ...) {
+  dataname <- as.character(substitute(data))  # "list" if no data=
   function.name <- "vlm"
 
   ocall <- match.call()
@@ -51,7 +50,7 @@ vlm <- function(formula,
 
 
   xlev <- .getXlevels(mt, mf)
-  y <- model.response(mf, "any") # model.extract(mf, "response")
+  y <- model.response(mf, "any")  # model.extract(mf, "response")
   x <- if (!is.empty.model(mt)) model.matrix(mt, mf, contrasts) else
        matrix(, NROW(y), 0)
   attr(x, "assign") <- attrassigndefault(x, mt)
@@ -92,43 +91,43 @@ vlm <- function(formula,
   }
 
   control <- control
-  Blist <- process.constraints(constraints, x, M)
+  Hlist <- process.constraints(constraints, x, M)
   intercept.only <- ncol(x) == 1 && dimnames(x)[[2]] == "(Intercept)"
 
-  fit <- vlm.wfit(xmat = x, zmat = y, Blist = Blist, wz = wz, U = NULL,
+  fit <- vlm.wfit(xmat = x, zmat = y, Hlist = Hlist, wz = wz, U = NULL,
                  matrix.out = FALSE, is.vlmX = FALSE,
-                 rss = TRUE, qr = qr.arg,
+                 ResSS = TRUE, qr = qr.arg,
                  x.ret = TRUE, offset = offset)
 
-  ncol_X_vlm <- fit$rank
-  fit$R <- fit$qr$qr[1:ncol_X_vlm, 1:ncol_X_vlm, drop = FALSE]
+  ncol.X.vlm <- fit$rank
+  fit$R <- fit$qr$qr[1:ncol.X.vlm, 1:ncol.X.vlm, drop = FALSE]
   fit$R[lower.tri(fit$R)] <- 0
 
 
 
 
-    fit$constraints <- Blist
+    fit$constraints <- Hlist
 
-    dnrow_X_vlm <- labels(fit$X_vlm)
-    xnrow_X_vlm <- dnrow_X_vlm[[2]]
+    dnrow.X.vlm <- labels(fit$X.vlm)
+    xnrow.X.vlm <- dnrow.X.vlm[[2]]
     dn <- labels(x)
     xn <- dn[[2]]
-    dX_vlm <- as.integer(dim(fit$X_vlm))
-    nrow_X_vlm <- dX_vlm[[1]]
-    ncol_X_vlm <- dX_vlm[[2]]
+    dX.vlm <- as.integer(dim(fit$X.vlm))
+    nrow.X.vlm <- dX.vlm[[1]]
+    ncol.X.vlm <- dX.vlm[[2]]
 
     misc <- list(
         colnames.x = xn,
-        colnames.X_vlm = xnrow_X_vlm,
+        colnames.X.vlm = xnrow.X.vlm,
         function.name = function.name,
         intercept.only=intercept.only,
         predictors.names = predictors.names,
         M = M,
         n = nrow(x),
-        nrow_X_vlm = nrow_X_vlm,
+        nrow.X.vlm = nrow.X.vlm,
         orig.assign = attr(x, "assign"),
         p = ncol(x),
-        ncol_X_vlm = ncol_X_vlm,
+        ncol.X.vlm = ncol.X.vlm,
         ynames = dimnames(y)[[2]])
     
     fit$misc <- misc
@@ -149,7 +148,7 @@ vlm <- function(formula,
       "coefficients" = fit$coefficients,
       "constraints"  = fit$constraints,
       "control"      = control, 
-      "criterion"    = list(deviance = fit$rss),
+      "criterion"    = list(deviance = fit$ResSS),
       "dispersion"   = 1,
       "df.residual"  = fit$df.residual,
       "df.total"     = n*M,
@@ -160,7 +159,7 @@ vlm <- function(formula,
       "R"            = fit$R,
       "rank"         = fit$rank,
       "residuals"    = as.matrix(fit$residuals),
-      "rss"          = fit$rss,
+      "ResSS"       = fit$ResSS,
       "smart.prediction" = as.list(fit$smart.prediction),
       "terms"        = list(terms = mt))
 
@@ -182,7 +181,7 @@ vlm <- function(formula,
   }
   if (x.arg)
     slot(answer, "x") <- x # The 'small' design matrix
-  if (control$save.weight)
+  if (control$save.weights)
     slot(answer, "weights") <- wz
   if (length(xlev))
     slot(answer, "xlevels") <- xlev
